#!/usr/bin/python

"""
Script to process the IRS Tax-Exempt files for a specific state
Frank Donnelly, Geospatial Data Librarian
francis.donnelly@baruch.cuny.edu
April 2014
This work is licensed under a
Creative Commons Attribution-NonCommercial-ShareAlike 3.0 United States License
http://creativecommons.org/licenses/by-nc-sa/3.0/us/
"""

import os.path

def process_irs(zipfile,monthyear,state,place):
    """Runs the program to process the IRS Tax-Exempt files. Four arguments as input:
zipfile - name of the file (with extension) that correlates ZIP codes with counties,
monthyear - (i.e. mar2014) to access data folders and attach date to output file name,
state - two-letter state code, to access the state-based csv file from the IRS,
place - name that describes general coverage area for the records that's appended to the output file name"""
    
    #FUNCTIONS
    def dictreader(file,delim):
        """Read a file that has two values per line as a dictionary"""
        dictionary={}
        readfile=open(file)
        for line in readfile:
            key,value=line.strip().split(delim)
            dictionary[key]=value
        readfile.close()
        return dictionary   

    def output_wlist(file,header,somelist):
        """Write a list out to a file, with a header row"""
        writefile=open(file,"w")
        writefile.writelines("\t".join(header)+"\n")  
        for records in somelist:
            writefile.writelines("\t".join(records)+"\n")
        writefile.close() 

    def addto_dict(somedict,somelist):
        """Add elements to a list value in a dictionary based on the number
        of items in another list"""    
        for values in somedict:
            somedict[values]=somedict[values]*len(somelist)

    #CREATE FILE FOR INDIVIDUAL RECORDS
    #Read in the ZIP and County codes for the area, then read in the matching records from
    #the IRS file, read and append the 5 digit ZIP and county code to the list, write
    #output to file

    zippath=os.path.join('z_geo',zipfile)   
    zipcounty=dictreader(zippath,',')

    taxpath=os.path.join('%s','eo_%s.csv') %(monthyear,state.lower())
    readfile=open(taxpath)
    keeplist=[]
    counter=0
    header=readfile.readline().strip().split(",")
    for line in readfile:
        record=line.strip().split(",")
        if any(v==record[6][0:5] for v in zipcounty.keys()):
            keeplist.append(record)
            counter=counter+1
    readfile.close()    

    for record in keeplist:
        record[2]=record[2].strip('% ')
        zip5=record[6][0:5]
        record.append(zip5)
        record.append(zipcounty.get(zip5))

    header.append('ZIP5')
    header.append('County')

    outpath=os.path.join('%s','irs_xorgs_%s_%s.txt') %(monthyear,monthyear,place)
    output_wlist(outpath,header,keeplist)

    print(counter,'records have been written to',outpath)

    #CREATE SUMMARY FILE
    #Calculate the unique number of counties being processed, modify a dictionary to hold
    #placeholder values for each county in a sorted list with the IRS subcodes as key, then loop
    #through the list and count records based on subcode and matching index number for
    #the county. Write output to file using the county name as a header 

    fips=set(zipcounty.values())
    counties=sorted(list(fips))

    eocodes=dict.fromkeys(['01','02','03','04','05','06','07','08','09','10','11','12',
                           '13','14','15','16','17','18','19','20','21','22','23','24',
                           '25','26','27','29','40','50','60','70','71','81','92',
                           'unknown'],[0])
    summary=dict.fromkeys(['Total'],[0])

    addto_dict(eocodes,counties)
    addto_dict(summary,counties)

    for record in keeplist:
        subcode=record[8]
        fips=record[29]    
        if subcode in eocodes:
            eocodes[subcode][counties.index(fips)]=eocodes[subcode][counties.index(fips)]+1
        else:
            eocodes['unknown'][counties.index(fips)]=eocodes['unknown'][counties.index(fips)]+1
        summary['Total'][counties.index(fips)]=summary['Total'][counties.index(fips)]+1

    countyname=dictreader('z_geo/county_name.txt','\t')

    sumheader=[]
    for fips in counties:
        sumheader.append(countyname.get(fips))
        
    sorted(sumheader)
    sumheader.insert(0,'EO SubCode')
    sumheader.insert(1,'Description')

    descriptions=dictreader('z_geo/irs_501.txt','\t')

    for key in descriptions:
        eocodes[key].insert(0,descriptions.get(key))
    summary['Total'].insert(0,'')

    outpath2=os.path.join('%s','irs_xcounts_%s_%s.txt') %(monthyear,monthyear,place)

    writefile=open(outpath2,'w')
    writefile.writelines(','.join(sumheader)+'\n')
    for key, values in sorted(eocodes.items()):
        writefile.writelines(', '.join([key,(str(values)).strip('[]')])+'\n')
    for key, values in (summary.items()):
        writefile.writelines(', '.join([key,(str(values)).strip('[]')])+'\n')
    writefile.close()

    print('Summary has been written to',outpath2)
